'A reference to the Phidget22.NET.dll must be added to new projects in VB.Net 
'From the "Project" menu select "Add Reference" 
'From the "Add Reference" window select the "Browse" tab 
'Browse to the directory you installed the Phidgets library to 
'The default is "C:\Program Files\Phidgets" 
'Select "Phidget22.NET.dll" and click "OK" 
'To view objects that have been added, select the "View" menu 
'Select "Object Browser" 
'Then expand the Phidget22.NET.dll item 



Imports Phidget22
Imports Phidget22.Events

Public Class Form1
    Dim WithEvents device As Phidget22.Magnetometer
    Dim open As ExampleUtils.ExampleUtils.CommandLineOpen
    Dim errorBox As ErrorEventBox
    Dim ambientMagneticField As Double = 0.57142 'Calgary, Alberta, Canada

    Public Sub New()
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
    End Sub

    Public Sub New(ByVal commandLine As String())
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        open.commandLine = commandLine
        Phidget22.Phidget.InvokeEventCallbacks = True

        ' This call is required by the designer.
        InitializeComponent()
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        errorBox = New ErrorEventBox(Me)
        device = New Phidget22.Magnetometer()

        initMagFieldGraph()

        open.openCmdLine(device)
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If device.Attached Then
            device.Close()
        End If
    End Sub

    Private Sub device_Attach(ByVal sender As Object, ByVal e As Phidget22.Events.AttachEventArgs) Handles device.Attach
        Dim attachedDevice As Magnetometer = CType(sender, Magnetometer)
        PhidgetInfoBox1.FillPhidgetInfo(attachedDevice)

        Try
            attachedDevice.ResetCorrectionParameters()

            dataIntervalTrk.Unit = "ms"
            dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString()
            dataIntervalTrk.Minimum = attachedDevice.MinDataInterval
            dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString()
            dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval
            dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString()
            dataIntervalTrk.Value = attachedDevice.DataInterval

            changeTriggerTrk.Unit = "G"
            changeTriggerTrk.minTxt = attachedDevice.MinMagneticFieldChangeTrigger.ToString()
            changeTriggerTrk.Minimum = attachedDevice.MinMagneticFieldChangeTrigger
            changeTriggerTrk.Maximum = attachedDevice.MaxMagneticFieldChangeTrigger
            changeTriggerTrk.maxTxt = attachedDevice.MaxMagneticFieldChangeTrigger.ToString()
            changeTriggerTrk.trkText = attachedDevice.MagneticFieldChangeTrigger.ToString()
            changeTriggerTrk.Value = attachedDevice.MagneticFieldChangeTrigger.ToString()
        Catch ex As Exception
            errorBox.addMessage("Error initializing device: " + ex.Message)
        End Try

        settingsBox.Visible = True
        outputBox.Visible = True
        magFieldGraphBox.Visible = True

    End Sub

    Private Sub device_Detach(ByVal sender As Object, ByVal e As Phidget22.Events.DetachEventArgs) Handles device.Detach
        PhidgetInfoBox1.Clear()

        settingsBox.Visible = False
        outputBox.Visible = False
        magFieldGraphBox.Visible = False

        calibrationStatusTxt.Visible = False
    End Sub

    Private Sub device_Error(ByVal sender As Object, ByVal e As Phidget22.Events.ErrorEventArgs) Handles device.Error
        errorBox.addMessage(e.Description)
    End Sub

    Private Sub device_FieldStrengthChange(ByVal sender As Object, ByVal e As Phidget22.Events.MagnetometerMagneticFieldChangeEventArgs) Handles device.MagneticFieldChange

        xaxistxt.Text = Math.Round(e.MagneticField(0), 3).ToString() + " G"
        yaxistxt.Text = Math.Round(e.MagneticField(1), 3).ToString() + " G"
        zaxistxt.Text = Math.Round(e.MagneticField(2), 3).ToString() + " G"

        timestamptxt.Text = e.Timestamp.ToString() + " ms"

        Try
            drawMagFieldGraph()
        Catch ex As Exception

        End Try
    End Sub

    Private Sub dataIntervalTrk_Scroll(sender As Object, e As EventArgs) Handles dataIntervalTrk.LogScroll
        If (MouseButtons = MouseButtons.Left) Then
            Try
                device.DataInterval = CType(dataIntervalTrk.Value, Integer)
            Catch ex As Exception
                errorBox.addMessage("Error setting data interval: " + ex.Message)
            End Try
        End If
    End Sub

    Private Sub dataIntervalTrk_LogMouseUp(sender As Object, e As EventArgs) Handles dataIntervalTrk.LogMouseUp
        Try
            device.DataInterval = CType(dataIntervalTrk.Value, Integer)
        Catch ex As Exception
            errorBox.addMessage("Error setting data interval: " + ex.Message)
        End Try
    End Sub

    Private Sub changeTriggertrk_Scroll(sender As Object, e As EventArgs) Handles changeTriggerTrk.LogScroll
        If (MouseButtons = MouseButtons.Left) Then
            Try
                device.MagneticFieldChangeTrigger = CType(changeTriggerTrk.Value, Integer)
            Catch ex As Exception
                errorBox.addMessage("Error setting data interval: " + ex.Message)
            End Try
        End If
    End Sub

    Private Sub changeTriggerTrk_LogMouseUp(sender As Object, e As EventArgs) Handles changeTriggerTrk.LogMouseUp
        Try
            device.MagneticFieldChangeTrigger = CType(changeTriggerTrk.Value, Integer)
        Catch ex As Exception
            errorBox.addMessage("Error setting data interval: " + ex.Message)
        End Try
    End Sub

    Private Sub setCompassParamsButton_Click(sender As Object, e As EventArgs) Handles setCompassParamsButton.Click
        Dim calibrationString As String = "0.41595, 0.02233, 0.15236, -0.22662, 2.36152, 2.27444, 2.57652, -0.03768, -0.01708, -0.03607, -0.00707, -0.01822, -0.00787" 'test string based on Calgary, AB and a prototype magnetometer

        Dim tib As ParameterInputBox = New ParameterInputBox(calibrationString)
        Dim result As DialogResult = tib.ShowDialog()

        If (result = DialogResult.OK) Then
            If (tib.parameters.Length > 0) Then
                calibrationString = tib.parameters
            Else
                calibrationStatusTxt.Text = "Un-Calibrated"
                calibrationStatusTxt.ForeColor = Color.Red
                calibrationStatusTxt.Visible = True
            End If
            Dim paramVals As String() = calibrationString.Split(New Char() {","})
            If (paramVals.Length = 13) Then
                Try
                    device.SetCorrectionParameters(Double.Parse(paramVals(0)),
                                                   Double.Parse(paramVals(1)),
                                                   Double.Parse(paramVals(2)),
                                                   Double.Parse(paramVals(3)),
                                                   Double.Parse(paramVals(4)),
                                                   Double.Parse(paramVals(5)),
                                                   Double.Parse(paramVals(6)),
                                                   Double.Parse(paramVals(7)),
                                                   Double.Parse(paramVals(8)),
                                                   Double.Parse(paramVals(9)),
                                                   Double.Parse(paramVals(10)),
                                                   Double.Parse(paramVals(11)),
                                                   Double.Parse(paramVals(12)))
                    ambientMagneticField = Double.Parse(paramVals(0))
                    calibrationStatusTxt.Text = "Calibrated"
                    calibrationStatusTxt.ForeColor = Color.Green
                    calibrationStatusTxt.Visible = True
                Catch ex As Exception
                    errorBox.addMessage("Erroir calibrating: " + ex.Message)
                End Try
            End If
        End If
    End Sub


#Region "graph"
    Dim magFieldGraphicsContext As BufferedGraphicsContext
    Dim magFieldGraphicsBuffer As BufferedGraphics
    Dim magFieldDrawingSurface As Bitmap

    Dim magFieldCircleRectangle, magFieldBoundsRectangle As Rectangle
    Dim magFieldXYAxisPen, magFieldCirclePen As Pen
    Dim magFieldCircleDiameter, magFieldCircleRadius As Double

    Private Sub initMagFieldGraph()
        'initialize our graphical representation of the compasseration data
        magFieldDrawingSurface = New Bitmap(magFieldView.ClientRectangle.Width, magFieldView.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb)

        magFieldBoundsRectangle = New Rectangle(0, 0, magFieldDrawingSurface.Width, magFieldDrawingSurface.Height)
        magFieldBoundsRectangle.Inflate(-1, -1)

        Dim xCenter As Double = CType(magFieldBoundsRectangle.Width / 2.0, Double)
        Dim yCenter As Double = CType(magFieldBoundsRectangle.Height / 2.0, Double)

        magFieldXYAxisPen = New Pen(Color.Black, 2)
        magFieldCirclePen = New Pen(Color.DarkBlue, 2)

        magFieldCircleDiameter = CType(magFieldDrawingSurface.Width / 1.1, Double)
        magFieldCircleRadius = magFieldCircleDiameter / 2

        magFieldCircleRectangle = New Rectangle(CType(Math.Round(xCenter - magFieldCircleRadius), Integer), CType(Math.Round(yCenter - magFieldCircleRadius), Integer), CType(Math.Round(magFieldCircleDiameter), Integer), CType(Math.Round(magFieldCircleDiameter), Integer))
    End Sub

    Private Sub drawMagFieldGraph()
        magFieldGraphicsContext = New BufferedGraphicsContext()
        magFieldGraphicsBuffer = magFieldGraphicsContext.Allocate(Graphics.FromImage(magFieldDrawingSurface), magFieldBoundsRectangle)
        magFieldGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
        magFieldGraphicsBuffer.Graphics.Clear(SystemColors.Control)

        Dim xCenter As Single = CType(magFieldBoundsRectangle.Width / 2.0, Single)
        Dim yCenter As Single = CType(magFieldBoundsRectangle.Height / 2.0, Single)

        Dim xOld As Single = xCenter + device.MagneticField(0) * magFieldCircleRadius / ambientMagneticField
        Dim yOld As Single = yCenter + device.MagneticField(1) * magFieldCircleRadius / ambientMagneticField

        magFieldGraphicsBuffer.Graphics.DrawLine(magFieldXYAxisPen, xCenter, yCenter, xOld, yOld)
        magFieldGraphicsBuffer.Graphics.DrawEllipse(magFieldCirclePen, magFieldCircleRectangle)

        If (device.MagneticField.Length = 3) Then
            Dim zOut As Double = device.MagneticField(2) * CType(1 / ambientMagneticField, Single)
            If (zOut > 0) Then
                magFieldGraphicsBuffer.Graphics.DrawEllipse(New Pen(Color.Red, 2),
                                                            New Rectangle(CType(xCenter, Integer) - CType(magFieldCircleRadius * zOut, Integer), CType(yCenter, Integer) - CType(magFieldCircleRadius * zOut, Integer), CType(magFieldCircleDiameter * zOut, Integer), CType(magFieldCircleDiameter * zOut, Integer)))
            Else
                magFieldGraphicsBuffer.Graphics.DrawEllipse(New Pen(Color.Green, 2),
                                                            New Rectangle(CType(xCenter, Integer) - CType(magFieldCircleRadius * -zOut, Integer), CType(yCenter, Integer) - CType(magFieldCircleRadius * -zOut, Integer), CType(magFieldCircleDiameter * -zOut, Integer), CType(magFieldCircleDiameter * -zOut, Integer)))

            End If
        End If

        magFieldGraphicsBuffer.Render(magFieldView.CreateGraphics())
        magFieldGraphicsBuffer.Dispose()
        magFieldGraphicsContext.Dispose()
    End Sub

#End Region

End Class



